@@ -23,7 +23,6 @@ class TestPerspectiveManager(object):
     def post(self, msg):
         '''boilerplate callback to simulate a client's `post()` method.'''
         msg = json.loads(msg)
-        print("self.post:", msg)
         assert msg["id"] is not None
 
     def validate_post(self, msg, expected=None):
@@ -42,22 +41,6 @@ class TestPerspectiveManager(object):
             "b": str
         }
 
-    def test_manager_host_view(self):
-        manager = PerspectiveManager()
-        table = Table(data)
-        view = table.view()
-        manager.host_view("view1", view)
-        assert manager.get_view("view1").to_dict() == data
-
-    def test_manager_host_table_or_view(self):
-        manager = PerspectiveManager()
-        table = Table(data)
-        view = table.view()
-        manager.host(table, name="table1")
-        manager.host(view, name="view1")
-        assert manager.get_table("table1").size() == 3
-        assert manager.get_view("view1").to_dict() == data
-
     def test_manager_host_invalid(self):
         manager = PerspectiveManager()
         with raises(PerspectiveError):
@@ -99,7 +82,7 @@ class TestPerspectiveManager(object):
             "b": str
         }
 
-        assert manager._tables["table1"]._index == "a"
+        assert manager._tables["table1"].get_index() == "a"
 
     def test_manager_create_indexed_table_and_update(self):
         message = {"id": 1, "name": "table1", "cmd": "table", "args": [data], "options": {"index": "a"}}
@@ -111,7 +94,7 @@ class TestPerspectiveManager(object):
             "a": int,
             "b": str
         }
-        assert manager._tables["table1"]._index == "a"
+        assert manager._tables["table1"].get_index() == "a"
         update_message = {"id": 2, "name": "table1", "cmd": "table_method", "method": "update", "args": [{"a": [1, 2, 3], "b": ["str1", "str2", "str3"]}]}
         manager._process(update_message, self.post)
         assert manager._tables["table1"].view().to_dict() == {
@@ -129,7 +112,7 @@ class TestPerspectiveManager(object):
             "a": int,
             "b": str
         }
-        assert manager._tables["table1"]._index == "a"
+        assert manager._tables["table1"].get_index() == "a"
         remove_message = {"id": 2, "name": "table1", "cmd": "table_method", "method": "remove", "args": [[1, 2]]}
         manager._process(remove_message, self.post)
         assert manager._tables["table1"].view().to_dict() == {
@@ -137,28 +120,13 @@ class TestPerspectiveManager(object):
             "b": ["c"]
         }
 
-    def test_manager_create_view(self):
-        message = {"id": 1, "name": "view1", "cmd": "view_method", "method": "schema", "args": []}
-        manager = PerspectiveManager()
-        table = Table(data)
-        view = table.view()
-        manager.host_table("table1", table)
-        manager.host_view("view1", view)
-        manager._process(message, self.post)
-        assert manager.get_view("view1").schema() == {
-            "a": int,
-            "b": str
-        }
-
     def test_locked_manager_create_view(self):
-        message = {"id": 1, "name": "view1", "cmd": "view_method", "method": "schema", "args": []}
+        message = {"id": 1, "table_name": "table1", "view_name": "view1", "cmd": "view"}
         manager = PerspectiveManager(lock=True)
         table = Table(data)
-        view = table.view()
         manager.host_table("table1", table)
-        manager.host_view("view1", view)
         manager._process(message, self.post)
-        assert manager.get_view("view1").schema() == {
+        assert manager._get_view("view1").schema() == {
             "a": int,
             "b": str
         }
@@ -329,26 +297,24 @@ class TestPerspectiveManager(object):
         message = {"id": 1, "name": "table1", "cmd": "table_method", "method": "schema", "args": [False]}
         manager = PerspectiveManager()
         table = Table(data)
-        view = table.view()
         manager.host_table("table1", table)
-        manager.host_view("view1", view)
         manager._process(message, post_callback)
 
     def test_manager_view_schema(self):
         post_callback = partial(self.validate_post, expected={
-            "id": 1,
+            "id": 2,
             "data": {
                 "a": "integer",
                 "b": "integer"
             }
         })
 
-        message = {"id": 1, "name": "view1", "cmd": "view_method", "method": "schema", "args": [False]}
+        make_view_message = {"id": 1, "table_name": "table1", "view_name": "view1", "cmd": "view", "config": {"row_pivots": ["a"]}}
+        message = {"id": 2, "name": "view1", "cmd": "view_method", "method": "schema", "args": [False]}
         manager = PerspectiveManager()
         table = Table(data)
-        view = table.view(row_pivots=["a"])
         manager.host_table("table1", table)
-        manager.host_view("view1", view)
+        manager._process(make_view_message, self.post)
         manager._process(message, post_callback)
 
     def test_manager_table_computed_schema(self):
@@ -378,7 +344,6 @@ class TestPerspectiveManager(object):
         table = Table(data)
         view = table.view()
         manager.host_table("table1", table)
-        manager.host_view("view1", view)
         manager._process(message, post_callback)
 
     def test_manager_table_get_computation_input_types(self):
@@ -396,31 +361,29 @@ class TestPerspectiveManager(object):
         }
         manager = PerspectiveManager()
         table = Table(data)
-        view = table.view()
         manager.host_table("table1", table)
-        manager.host_view("view1", view)
         manager._process(message, post_callback)
 
     def test_manager_view_computed_schema(self):
         post_callback = partial(self.validate_post, expected={
-            "id": 1,
+            "id": 2,
             "data": {
                 "abc": "float"
             }
         })
 
-        message = {"id": 1, "name": "view1", "cmd": "view_method", "method": "computed_schema", "args": [False]}
-        manager = PerspectiveManager()
-        table = Table(data)
-        view = table.view(computed_columns=[
+        make_view_message = {"id": 1, "table_name": "table1", "view_name": "view1", "cmd": "view", "config": {"computed_columns": [
             {
                 "column": "abc",
                 "computed_function_name": "+",
                 "inputs": ["a", "a"]
             }
-        ])
+        ]}}
+        message = {"id": 2, "name": "view1", "cmd": "view_method", "method": "computed_schema", "args": [False]}
+        manager = PerspectiveManager()
+        table = Table(data)
         manager.host_table("table1", table)
-        manager.host_view("view1", view)
+        manager._process(make_view_message, self.post)
         manager._process(message, post_callback)
 
     # serialization
@@ -955,7 +918,7 @@ class TestPerspectiveManager(object):
         make_view = {"id": 2, "table_name": "table1", "view_name": "view1", "cmd": "view"}
         manager._process(make_view, self.post)
 
-        view = manager.get_view("view1")
+        view = manager._get_view("view1")
         view.on_delete(delete_callback)
 
         delete_view = {"id": 3, "name": "view1", "cmd": "view_method", "method": "delete"}
@@ -977,7 +940,7 @@ class TestPerspectiveManager(object):
         make_view = {"id": 2, "table_name": "table1", "view_name": "view1", "cmd": "view"}
         manager._process(make_view, self.post)
 
-        view = manager.get_view("view1")
+        view = manager._get_view("view1")
         view.on_delete(delete_callback)
         view.remove_delete(delete_callback)
 
@@ -998,29 +961,29 @@ class TestPerspectiveManager(object):
             "a": [1, 2, 3, 4, 5, 6]
         }
 
-        def fake_queue_process(table_id, state_manager):
+        def fake_queue_process(f, *args, **kwargs):
             s.set(s.get() + 1)
-            state_manager.call_process(table_id)
+            f(*args, **kwargs)
 
-        manager._set_queue_process(fake_queue_process)
+        manager.set_loop_callback(fake_queue_process)
         table.update({"a": [7, 8, 9]})
-        assert s.get() == 1
+        assert s.get() == 2
 
     def test_manager_set_queue_process_before_host_table(self, sentinel):
         s = sentinel(0)
         manager = PerspectiveManager()
         table = Table({"a": [1, 2, 3]})
 
-        def fake_queue_process(table_id, state_manager):
+        def fake_queue_process(f, *args, **kwargs):
             s.set(s.get() + 1)
-            state_manager.call_process(table_id)
+            f(*args, **kwargs)
 
-        manager._set_queue_process(fake_queue_process)
+        manager.set_loop_callback(fake_queue_process)
         manager.host_table("tbl", table)
         table.update({"a": [4, 5, 6]})
         table.update({"a": [4, 5, 6]})
 
-        assert s.get() == 2
+        assert s.get() == 3
 
     def test_manager_set_queue_process_multiple(self, sentinel):
         # manager2's queue process should not affect manager1,
@@ -1034,11 +997,11 @@ class TestPerspectiveManager(object):
         manager.host_table("tbl", table)
         manager2.host_table("tbl2", table2)
 
-        def fake_queue_process(table_id, state_manager):
+        def fake_queue_process(f, *args, **kwargs):
             s2.set(s2.get() + 1)
-            state_manager.call_process(table_id)
+            f(*args, **kwargs)
 
-        manager2._set_queue_process(fake_queue_process)
+        manager2.set_loop_callback(fake_queue_process)
 
         table.update({"a": [4, 5, 6]})
         assert table.view().to_dict() == {
@@ -1055,4 +1018,4 @@ class TestPerspectiveManager(object):
             "a": [1, 2, 3, 7, 8, 9]
         }
         assert s.get() == 0
-        assert s2.get() == 1
+        assert s2.get() == 2
