@@ -6,11 +6,13 @@
 # the Apache License 2.0.  The full license can be found in the LICENSE file.
 #
 
+import random
 import pandas as pd
 import numpy as np
+from perspective import PerspectiveCppError
 from perspective.table import Table
 from datetime import date, datetime
-from pytest import mark
+from pytest import mark, raises
 
 
 def compare_delta(received, expected):
@@ -821,7 +823,7 @@ class TestView(object):
         view.on_update(cb2)
         view.on_update(cb1)
         view.remove_update(cb1)
-        assert len(view._callbacks) == 1
+        assert len(view._update_callbacks) == 1
         tbl.update(data)
         assert s.get() == 2
 
@@ -842,6 +844,23 @@ class TestView(object):
         view.on_update(cb1, mode="row")
         tbl.update(update_data)
 
+    def test_view_row_delta_zero_column_subset(self, util):
+        data = [{"a": 1, "b": 2}, {"a": 3, "b": 4}]
+        update_data = {
+            "a": [5],
+            "b": [6]
+        }
+
+        def cb1(port_id, delta):
+            compare_delta(delta, {
+                "b": [6]
+            })
+
+        tbl = Table(data)
+        view = tbl.view(columns=["b"])
+        view.on_update(cb1, mode="row")
+        tbl.update(update_data)
+
     def test_view_row_delta_zero_from_schema(self, util):
         update_data = {
             "a": [5],
@@ -859,6 +878,26 @@ class TestView(object):
         view.on_update(cb1, mode="row")
         tbl.update(update_data)
 
+    def test_view_row_delta_zero_from_schema_column_subset(self, util):
+        update_data = {
+            "a": [5],
+            "b": [6]
+        }
+
+        def cb1(port_id, delta):            
+            compare_delta(delta, {
+                "b": [6]
+            })
+
+        tbl = Table({
+            "a": int,
+            "b": int
+        })
+
+        view = tbl.view(columns=["b"])
+        view.on_update(cb1, mode="row")
+        tbl.update(update_data)
+
     def test_view_row_delta_zero_from_schema_filtered(self, util):
         update_data = {
             "a": [8, 9, 10, 11],
@@ -1261,3 +1300,308 @@ class TestView(object):
         tbl = Table(data)
         view = tbl.view(columns=["a"], sort=[["b", "desc"]])
         assert view._num_hidden_cols() == 1
+
+    def test_view_context_two_update_clears_column_regression(self, util):
+        """Tests that, when a 2-sided View() is updated to a state where one of
+        the column groups is empty, an infinite loop is not encountered.
+        """
+        data = [
+            {"a": "a", "b": 1, "c": 1.5, "i": 0},
+            {"a": "a", "b": 2, "c": 2.5, "i": 1},
+            {"a": "a", "b": 3, "c": 3.5, "i": 2},
+            {"a": "b", "b": 1, "c": 4.5, "i": 3},
+            {"a": "b", "b": 2, "c": 5.5, "i": 4},
+            {"a": "b", "b": 3, "c": 6.5, "i": 5},
+        ]
+
+        tbl = Table(data, index="i")
+        view = tbl.view(
+            row_pivots=["b"],
+            column_pivots=["a"],
+            columns=["c"],
+            filter=[["c", ">", 0]],
+            sort=[["c", "asc"], ["a", "col asc"]],
+        )
+
+        assert view.to_records() == [
+             {'__ROW_PATH__': [], 'a|c': 7.5, 'b|c': 16.5},
+             {'__ROW_PATH__': [1], 'a|c': 1.5, 'b|c': 4.5},
+             {'__ROW_PATH__': [2], 'a|c': 2.5, 'b|c': 5.5},
+             {'__ROW_PATH__': [3], 'a|c': 3.5, 'b|c': 6.5}
+        ]
+
+        tbl.update(
+            [
+                {"c": -1, "i": 0},
+                {"c": -1, "i": 1},
+                {"c": -1, "i": 2},
+            ]
+        )
+
+        assert view.to_records() == [
+            {"__ROW_PATH__": [], "b|c": 16.5},
+            {"__ROW_PATH__": [1], "b|c": 4.5},
+            {"__ROW_PATH__": [2], "b|c": 5.5},
+            {"__ROW_PATH__": [3], "b|c": 6.5},
+        ]
+
+        tbl.update(
+            [
+                {"a": "a", "b": 1, "c": 1.5, "i": 6},
+                {"a": "a", "b": 2, "c": 2.5, "i": 7},
+                {"a": "a", "b": 3, "c": 3.5, "i": 8},
+            ]
+        )
+
+        assert view.to_records() == [
+             {'__ROW_PATH__': [], 'a|c': 7.5, 'b|c': 16.5},
+             {'__ROW_PATH__': [1], 'a|c': 1.5, 'b|c': 4.5},
+             {'__ROW_PATH__': [2], 'a|c': 2.5, 'b|c': 5.5},
+             {'__ROW_PATH__': [3], 'a|c': 3.5, 'b|c': 6.5}
+        ]
+
+        assert tbl.size() == 9
+
+    # expand/collapse
+
+    def test_view_collapse_one(self):
+        data = [{"a": 1, "b": 2}, {"a": 3, "b": 4}]
+        tbl = Table(data)
+        view = tbl.view(row_pivots=["a"])
+        assert view.collapse(0) == 2
+
+    def test_view_collapse_two(self):
+        data = [{"a": 1, "b": 2, "c": "a"}, {"a": 3, "b": 4, "c": "b"}]
+        tbl = Table(data)
+        view = tbl.view(row_pivots=["a"], column_pivots=["c"])
+        assert view.collapse(0) == 0
+
+    def test_view_collapse_two_column_only(self):
+        data = [{"a": 1, "b": 2, "c": "a"}, {"a": 3, "b": 4, "c": "b"}]
+        tbl = Table(data)
+        view = tbl.view(column_pivots=["c"])
+        assert view.collapse(0) == 0
+
+    def test_view_expand_one(self):
+        data = [{"a": 1, "b": 2}, {"a": 3, "b": 4}]
+        tbl = Table(data)
+        view = tbl.view(row_pivots=["a"])
+        assert view.expand(0) == 0
+
+    def test_view_expand_two(self):
+        data = [{"a": 1, "b": 2, "c": "a"}, {"a": 3, "b": 4, "c": "b"}]
+        tbl = Table(data)
+        view = tbl.view(row_pivots=["a"], column_pivots=["c"])
+        assert view.expand(1) == 1
+
+    def test_view_expand_two_column_only(self):
+        data = [{"a": 1, "b": 2, "c": "a"}, {"a": 3, "b": 4, "c": "b"}]
+        tbl = Table(data)
+        view = tbl.view(column_pivots=["c"])
+        assert view.expand(0) == 0
+
+    # view config validation
+
+    def test_invalid_column_should_throw(self):
+        data = [{"a": 1, "b": 2, "c": "a"}, {"a": 3, "b": 4, "c": "b"}]
+        tbl = Table(data)
+        with raises(PerspectiveCppError) as ex:
+            tbl.view(columns=["x"])
+        assert str(ex.value) == "Invalid column 'x' found in View columns.\n"
+
+    def test_invalid_column_should_throw_and_updates_should_work(self):
+        data = [{"a": 1, "b": 2, "c": "a"}, {"a": 3, "b": 4, "c": "b"}]
+        tbl = Table(data)
+
+        with raises(PerspectiveCppError) as ex:
+            tbl.view(columns=["x"])
+        assert str(ex.value) == "Invalid column 'x' found in View columns.\n"
+
+        for i in range(100):
+            tbl.update(data)
+            # force call to _process which should shake out invalid column ptrs
+            tbl.size()
+
+        view2 = tbl.view()
+        assert view2.num_rows() == 202
+
+    def test_invalid_column_aggregate_should_throw(self):
+        data = [{"a": 1, "b": 2, "c": "a"}, {"a": 3, "b": 4, "c": "b"}]
+        tbl = Table(data)
+
+        with raises(PerspectiveCppError) as ex:
+            tbl.view(columns=["x"], aggregates={"x": "sum"})
+
+        assert str(ex.value) == "Invalid column 'x' found in View columns.\n"
+
+    def test_invalid_column_aggregate_should_throw_and_updates_should_work(self):
+        data = [{"a": 1, "b": 2, "c": "a"}, {"a": 3, "b": 4, "c": "b"}]
+        tbl = Table(data)
+
+        with raises(PerspectiveCppError) as ex:
+            tbl.view(columns=["x"], aggregates={"x": "sum"})
+
+        assert str(ex.value) == "Invalid column 'x' found in View columns.\n"
+
+        for i in range(100):
+            tbl.update(data)
+            # force call to _process which should shake out invalid column ptrs
+            tbl.size()
+
+        view2 = tbl.view()
+        assert view2.num_rows() == 202
+
+    def test_invalid_row_pivot_should_throw(self):
+        data = [{"a": 1, "b": 2, "c": "a"}, {"a": 3, "b": 4, "c": "b"}]
+        tbl = Table(data)
+        with raises(PerspectiveCppError) as ex:
+            tbl.view(row_pivots=["x"])
+        assert str(ex.value) == "Invalid column 'x' found in View row_pivots.\n"
+
+    def test_invalid_column_pivot_should_throw(self):
+        data = [{"a": 1, "b": 2, "c": "a"}, {"a": 3, "b": 4, "c": "b"}]
+        tbl = Table(data)
+        with raises(PerspectiveCppError) as ex:
+            tbl.view(column_pivots=["x"])
+        assert str(ex.value) == "Invalid column 'x' found in View column_pivots.\n"
+
+    def test_invalid_filters_should_throw(self):
+        data = [{"a": 1, "b": 2, "c": "a"}, {"a": 3, "b": 4, "c": "b"}]
+        tbl = Table(data)
+        with raises(PerspectiveCppError) as ex:
+            tbl.view(filter=[["x", "==", "abc"]])
+        assert str(ex.value) == "Could not get dtype for column `x` as it does not exist in the schema.\n"
+
+    def test_invalid_sorts_should_throw(self):
+        data = [{"a": 1, "b": 2, "c": "a"}, {"a": 3, "b": 4, "c": "b"}]
+        tbl = Table(data)
+        with raises(PerspectiveCppError) as ex:
+            tbl.view(sort=[["x", "desc"]])
+        assert str(ex.value) == "Invalid column 'x' found in View sorts.\n"
+
+    def test_should_throw_on_first_invalid(self):
+        data = [{"a": 1, "b": 2, "c": "a"}, {"a": 3, "b": 4, "c": "b"}]
+        tbl = Table(data)
+        with raises(PerspectiveCppError) as ex:
+            tbl.view(row_pivots=["a"], column_pivots=["c"], filter=[["a", ">", 1]], aggregates={"a": "avg"}, sort=[["x", "desc"]])
+        assert str(ex.value) == "Invalid column 'x' found in View sorts.\n"
+
+    def test_invalid_columns_not_in_computed_should_throw(self):
+        data = [{"a": 1, "b": 2, "c": "a"}, {"a": 3, "b": 4, "c": "b"}]
+        tbl = Table(data)
+        with raises(PerspectiveCppError) as ex:
+            tbl.view(
+                columns=["x"],
+                computed_columns=[
+                    {
+                        "column": "abc",
+                        "computed_function_name": "exp",
+                        "inputs": ["a"]
+                    }
+                ]
+            )
+        assert str(ex.value) == "Invalid column 'x' found in View columns.\n"
+
+    def test_should_not_throw_valid_computed_columns(self):
+        data = [{"a": 1, "b": 2, "c": "a"}, {"a": 3, "b": 4, "c": "b"}]
+        tbl = Table(data)
+        view = tbl.view(
+            columns=["abc"],
+            computed_columns=[
+                {
+                    "column": "abc",
+                    "computed_function_name": "uppercase",
+                    "inputs": ["c"]
+                }
+            ]
+        )
+
+        assert view.schema() == {
+            "abc": str
+        }
+
+    def test_should_not_throw_valid_computed_columns_config(self):
+        data = [{"a": 1, "b": 2, "c": "a"}, {"a": 3, "b": 4, "c": "b"}]
+        tbl = Table(data)
+        view = tbl.view(
+            aggregates={
+                "abc": "dominant"
+            },
+            columns=["abc"],
+            sort=[["abc", "desc"]],
+            filter=[["abc", "==", "A"]],
+            row_pivots=["abc"],
+            column_pivots=["abc"],
+            computed_columns=[
+                {
+                    "column": "abc",
+                    "computed_function_name": "uppercase",
+                    "inputs": ["c"]
+                }
+            ]
+        )
+
+        assert view.schema() == {
+            "abc": str
+        }
+
+    def test_should_not_throw_ancestor_computed(self):
+        cc = [
+            {
+                "column": "exp(Discount)",
+                "computed_function_name": "exp",
+                "inputs": ["Discount"],
+            },
+            {
+                "column": "(Profit * exp(Discount))",
+                "computed_function_name": "*",
+                "inputs": ["Profit", "exp(Discount)"],
+            },
+            {
+                "column": "(Sales + (Profit * exp(Discount)))",
+                "computed_function_name": "+",
+                "inputs": ["Sales", "(Profit * exp(Discount))"],
+            },
+            {
+                "column": "(Profit * Discount)",
+                "computed_function_name": "*",
+                "inputs": ["Profit", "Discount"],
+            },
+            {
+                "column": "sqrt(Row ID)",
+                "computed_function_name": "sqrt",
+                "inputs": ["Row ID"],
+            },
+            {
+                "column": "((Profit * Discount) * sqrt(Row ID))",
+                "computed_function_name": "*",
+                "inputs": ["(Profit * Discount)", "sqrt(Row ID)"],
+            },
+            {
+                "column": "((Sales + (Profit * exp(Discount))) - ((Profit * Discount) * sqrt(Row ID)))",
+                "computed_function_name": "-",
+                "inputs": [
+                    "(Sales + (Profit * exp(Discount)))",
+                    "((Profit * Discount) * sqrt(Row ID))",
+                ],
+            },
+        ]
+
+        table = Table({
+            "Discount": [random.random() for i in range(10)],
+            "Profit": [random.random() for i in range(10)],
+            "Row ID": [random.random() for i in range(10)],
+            "Sales": [random.random() for i in range(10)],
+        })
+
+        view = table.view(
+            columns=["sqrt(Row ID)"],
+            filter=[["((Profit * Discount) * sqrt(Row ID))", ">", 0]],
+            sort=[["exp(Discount)", "desc"]],
+            aggregates={"sqrt(Row ID)": "avg"},
+            computed_columns=cc,
+            row_pivots=["((Sales + (Profit * exp(Discount))) - ((Profit * Discount) * sqrt(Row ID)))"],
+            column_pivots=["((Sales + (Profit * exp(Discount))) - ((Profit * Discount) * sqrt(Row ID)))"]
+        )
+
+        assert view.num_columns() == 20
